/*
 * ARMPL version 21.0 Copyright ARM, NAG 2021
 */

#ifndef ARMPL_STATUS_H
#define ARMPL_STATUS_H

#ifdef __cplusplus
extern "C" {
#endif

/* ENUMs */

typedef enum armpl_status {
	ARMPL_STATUS_SUCCESS=0,
	ARMPL_STATUS_INPUT_PARAMETER_ERROR=1,
	ARMPL_STATUS_EXECUTION_FAILURE=2,
} armpl_status_t;

#ifdef __cplusplus
} //extern "C"
#endif

#endif //ARMPL_STATUS_H
