!
!   ARMPL version 21.0 Copyright ARM, NAG 2021
!
!   Interface blocks for Fortran routines in ARMPL
!
    Module armpl_library
!     .. Use Statements ..
      Use armpl_blas
      Use armpl_lapack
      Use armpl_sparse
!     .. Interface Blocks ..
      Interface
        Subroutine armplinfo
        End Subroutine armplinfo
        Subroutine armplversion(major, minor, patch)
!     .. Implicit None Statement ..
          Implicit None
!     .. Scalar Arguments ..
          Integer :: major, minor, patch
        End Subroutine armplversion
      End Interface
    End Module armpl_library
