/*
 * ARMPL version 21.0 Copyright ARM, NAG 2021
 */

#ifndef ARMPL_INT_T
#define ARMPL_INT_T

#ifdef INTEGER64
typedef long armpl_int_t;
typedef unsigned long armpl_uint_t;
#else
typedef int armpl_int_t;
typedef unsigned int armpl_uint_t;
#endif
typedef int armpl_strlen_t;

#endif
